'use strict';

var cac = require('cac');
var consola = require('consola');
var colors = require('picocolors');
var variableGen = require('./variable-gen-BVqVtmyb.js');
require('node:fs/promises');
require('node:url');
require('json-stringify-pretty-compact');
require('pathe');
require('@evan/concurrency');
require('stylis');
require('@octokit/core');
require('node:fs');
require('zod');
require('linkedom');
require('playwright');
require('deepmerge');

var version = "6.0.3";

const cli = cac.cac("gfm");
cli.command("generate [key]", "Fetch parsing metadata for all fonts").option("-n, --normal", "Fetch only normal Google Fonts Developer API").option("-v, --variable", "Fetch only variable metadata").action(async (key, options) => {
  try {
    const finalKey = key ?? process.env.API_KEY;
    if (options.normal) {
      consola.consola.info("Fetching Google Fonts Developer API...");
      await variableGen.fetchAPI(finalKey);
    } else if (options.variable) {
      consola.consola.info("Fetching Google Fonts Variable Data...");
      await variableGen.fetchVariable();
    } else {
      consola.consola.info("Fetching all Google Fonts Data...");
      await Promise.all([variableGen.fetchAPI(finalKey), variableGen.fetchVariable()]);
    }
  } catch (error) {
    consola.consola.error(error);
    process.exit(1);
  }
});
cli.command("parse [key]", "Process metadata for v1 and v2 from gfm generate").option("-1, --v1", "Only parse v1 metadata").option("-2, --v2", "Only parse v2 metadata").option("-r, --axis-registry", "Only parse axis registry metadata").option("-v, --variable", "Only parse variable metadata").option("-i, --icon", "Only parse icon metadata").option("-l, --license", "Only parse license metadata").option("-f, --force", "Skip cache and force parse all metadata").option("--no-validate", "Skip validating metadata result with schema").action(async (key, options) => {
  try {
    const force = options.force ?? false;
    const noValidate = options["no-validate"] ?? false;
    if (options.v1) {
      if (options.force) {
        consola.consola.info(
          `Parsing v1 metadata... ${colors.bold(colors.red("[FORCE]"))}`
        );
      } else {
        consola.consola.info("Parsing v1 metadata...");
      }
      await variableGen.parsev1(force, noValidate);
    }
    if (options.v2) {
      if (options.force) {
        consola.consola.info(
          `Parsing v2 metadata... ${colors.bold(colors.red("[FORCE]"))}`
        );
      } else {
        consola.consola.info("Parsing v2 metadata...");
      }
      await variableGen.parsev2(force, noValidate);
    }
    if (options.axisRegistry) {
      consola.consola.info("Parsing axis registry metadata...");
      await variableGen.generateAxis(key);
    }
    if (options.variable) {
      consola.consola.info("Parsing variable metadata...");
      await variableGen.parseVariable(noValidate);
    }
    if (options.icon) {
      if (options.force) {
        consola.consola.info(
          `Parsing icon metadata... ${colors.bold(colors.red("[FORCE]"))}`
        );
      } else {
        consola.consola.info("Parsing icon metadata...");
      }
      await variableGen.parseIcons(force);
    }
    if (options.license) {
      consola.consola.info("Parsing license metadata...");
      await variableGen.parseLicenses();
    }
    if (!options.v1 && !options.v2 && !options.variable && !options.icon && !options.license && !options.axisRegistry) {
      if (options.force) {
        consola.consola.info(
          `Parsing all metadata... ${colors.bold(colors.red("[FORCE]"))}`
        );
      } else {
        consola.consola.info("Parsing all metadata...");
      }
      await variableGen.parsev1(force, noValidate);
      await variableGen.parsev2(force, noValidate);
      await variableGen.generateAxis(key);
      await variableGen.parseVariable(noValidate);
      await variableGen.parseIcons(force);
      await variableGen.parseLicenses();
    }
  } catch (error) {
    consola.consola.error(error);
    process.exit(1);
  }
});
cli.command("validate", "Validate stored metadata with schema.").option("-1, --v1", "Only validate APIv1 metadata").option("-2, --v2", "Only validate APIv2 metadata").option("--variable", "Only validate variable metadata").action((options) => {
  try {
    if (options.v1) variableGen.validateCLI("v1");
    if (options.v2) variableGen.validateCLI("v2");
    if (options.variable) variableGen.validateCLI("variable");
    if (!options.v1 && !options.v2 && !options.variable) {
      variableGen.validateCLI("v1");
      variableGen.validateCLI("v2");
      variableGen.validateCLI("variable");
    }
  } catch (error) {
    consola.consola.error(error);
    process.exit(1);
  }
});
cli.help();
cli.version(version);
cli.parse();
